#!/usr/bin/env python2

import numpy as np
import cv2
import time

def detect_red_blob(frame, kernel, max_fun):
    #Detect Red
    rows = frame.shape[0]
    cols = frame.shape[1]
    kernel_size = int(kernel.shape[0]/2)
    hsv = cv2.cvtColor(frame,cv2.COLOR_BGR2HSV)
    lower_red = np.array([-20, 100, 100])
    upper_red = np.array([13,  255, 255])
    mask_red = cv2.inRange(hsv, lower_red, upper_red)
    g_red = cv2.GaussianBlur(mask_red, (31, 31), 0)
    # g_red = cv2.GaussianBlur(mask_red, (11, 11), 0)
    (minVal, maxVal, minLoc, i) = cv2.minMaxLoc(g_red)

    if i[0]>kernel_size and i[1]>kernel_size and i[0]<cols-kernel_size and i[1]<rows-kernel_size:
        max_fun[i[1]-kernel_size:i[1]+kernel_size,
                i[0]-kernel_size:i[0]+kernel_size] += 0.5*kernel
    return max_fun

def create_kernel(kernel_size = 10):
    kernel = np.zeros((2*kernel_size,2*kernel_size))
    for i in range(2*kernel_size):
        for j in range(2*kernel_size):
            kernel[i,j] = max(0,kernel_size-(np.sqrt(pow(kernel_size-i,2)+pow(kernel_size-j,2))))
    return kernel

def main_function():
    cv2.namedWindow("preview")
    vc = cv2.VideoCapture(0)
    start = time.time()
    kernel = create_kernel()
    
    if vc.isOpened(): # try to get the first frame
        rval, frame = vc.read()
        rows = frame.shape[0]
        cols = frame.shape[1]
        max_fun = np.zeros((rows,cols))
    else:
        rval = False
    
    while rval:
        cv2.imshow("preview", frame)
        rval, frame = vc.read()
        gray = cv2.cvtColor(frame,cv2.COLOR_BGR2GRAY)
    
        max_fun = np.zeros((rows,cols))
    
        for r in [20, 30, 40, 50, 60, 70, 80, 90, 100]:
            circles = cv2.HoughCircles(gray, cv2.HOUGH_GRADIENT, 1, r,
                             param1=100, param2=40,
                             minRadius=r, maxRadius=r+10)
            if circles is not None:
                circles = np.uint16(np.around(circles))
                for i in circles[0, :]:
                    center = (i[0], i[1])
                    cv2.circle(frame, center, 1, (r*2, 255-r*2, 255-r*2), 3)
                    radius = i[2]
                    cv2.circle(frame, center, radius, (r*2, 255-r*2, 255-r*2), 1)
                    #add kernel at the center of the circle to the objective function
                    kernel_size = int(kernel.shape[0]/2)
                    if (i[0]>kernel_size
                            and i[1]>kernel_size
                            and i[0]<cols-kernel_size
                            and i[1]<rows-kernel_size):
                        max_fun[i[1]-kernel_size:i[1]+kernel_size,
                                i[0]-kernel_size:i[0]+kernel_size] += kernel
    
        #uncomment to enable red-color-detection
        # max_fun = detect_red_blob(frame, kernel, max_fun)
    
        (minVal, maxVal, minLoc, maxLoc) = cv2.minMaxLoc(max_fun)
        cv2.circle(frame, maxLoc, 31, (5, 0, 255), 1)
    
        #draw yellow line at retrieved x-value
        cv2.line(frame,(maxLoc[0],0),(maxLoc[0],rows),(0,255,250),2)
        cv2.line(frame,(0,maxLoc[1]),(cols,maxLoc[1]),(0,255,250),2)
    
        # framerate
        end = time.time()
        seconds = end - start
        start = time.time()
        fps  = 1 / seconds
        print("offcenter:", maxLoc[0]-cols/2, "fps:", fps)
    
        key = cv2.waitKey(20)
        if key == 27: # exit on ESC
            break
    
    cv2.destroyWindow("preview")
    vc.release()

if __name__ == "__main__":    
    main_function()

#using cv2.SimpleBlobDetector_create would be more robust agains ellipticity
